#include "stdAfx.h"
#include "Formations.h"

//////////////////////////////////////////////////////////////////////////////////
//						Team Position
//////////////////////////////////////////////////////////////////////////////////

CTeamPosition::CTeamPosition()
{
  m_dHomeBuffer = 0;
  m_type = PT_None;
  m_side = PS_None;
}


void CTeamPosition::SetHome(C2DVector p, float buf, Ptype t, Pside s)
{
  m_Home = p;
  m_dHomeBuffer = buf;

  m_type = t; 
  m_side = s;
}


void CTeamPosition::SetHomeRange(C2DVector center, C2DVector size)
{
  m_HomeRange = RRectangle(center,size);
}


void CTeamPosition::SetMaxRange (C2DVector center, C2DVector size)
{
  m_MaxRange = RRectangle(center,size);
}

//////////////////////////////////////////////////////////////////////////////////
//                        Unit Class                                            
//////////////////////////////////////////////////////////////////////////////////

CUnit::CUnit()
{
  m_type = UT_None;
  m_size = captain = 0;
  members = NULL;
}


CUnit::~CUnit()
{
  if ( members != NULL )
    delete [] members;
}


void CUnit::InitializeUnit(Utype tp, int sz, Pnum *mems, Pnum capt)
{
  m_type = tp;
  m_size = sz;
  captain = mems[capt];
  members = new Pnum[m_size];

  for (int i=0; i<m_size; i++)
    members[i] = mems[i];
}


bool CUnit::IsMember(Pnum position)
{
  for (int i=0; i<m_size; i++){
    if ( members[i] == position )
      return TRUE;
  }
  return FALSE;
}


bool CUnit::IsCaptain(Pnum position)
{
  if ( members[0] == position )
      return TRUE;
  return FALSE;
}


Pnum CUnit::ImportantPosition(Pnum p1, Pnum p2)
{
  int index1, index2;

  for (int i=0; i<m_size; i++){
    if ( members[i] == p1 )
      index1 = i;
    if ( members[i] == p2 )
      index2 = i;
  }

  if ( index1 < index2 )
    return p1;
  else if ( index2 < index1 )
    return p2;
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////////
//							Formation Class
//////////////////////////////////////////////////////////////////////////////////

CFormation::CFormation(Ftype formation_number, int sz)
{
  m_type = formation_number;
  m_size = sz;
  m_FormationUnits = NULL;

  for (int i=1; i<=m_size; i++)            /* Positions start at 1            */
    m_ReceiverList[i] = new Pnum[m_size];
                                         /* receiver positions              */
  m_TempReceiverList = new Pnum[m_size];     /* Used to pass back possible receivers */
}


CFormation::~CFormation()
{
  if ( m_FormationUnits != NULL )
    delete [] m_FormationUnits;

  for (int i=1; i<=m_size; i++)
    delete [] m_ReceiverList[i];
  delete [] m_TempReceiverList;
}

void CFormation::AllocateUnits(int num)
{
  m_nNumUnits = num;
  m_FormationUnits = new CUnit[m_nNumUnits];
}


void CFormation::SetPosition(Pnum num, C2DVector home, float buffer,
			    C2DVector home_center, C2DVector home_size,
			    C2DVector max_center,  C2DVector max_size,
			    Ptype t, Pside s)
{
  /* Use this to put in the parameters for the positions in a given formation */
  m_PositionList[num].SetHome(home,buffer,t,s);
  m_PositionList[num].SetHomeRange(home_center,home_size);
  m_PositionList[num].SetMaxRange(max_center,max_size);
}


CTeamPosition *CFormation::GetPosition(Pnum num)
{
//  if (num > size || num < 1) 
    //my_error("position list index only goes from 1 to size %d (trying to get %d)",size,num);
  return &(m_PositionList[num]); 
}


void CFormation::SetCandidateReceivers(Pnum position, int num, Pnum *PrefList)
{
  int i;

  for (i=0; i<num; i++)
    m_ReceiverList[position][i] = PrefList[i];

  for (; i<m_size; i++)
    m_ReceiverList[position][i] = Pnum_Unknown;  /* End the list with sequence       */
}					       /* of unknowns----means no reciever */


Pnum *CFormation::GetCandidateReceivers(Pnum position, Fside LocationSide)
{
  /* Either call a function to build the list on the fly, 
     or return the precanned ReceiverList */

  switch(m_type){
  case FT_433: 
    Build_433_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_442: 
    Build_442_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_352: 
  //  Build_352_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_72: 
  //  Build_72_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_334: 
  //  Build_334_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_244: 
  //  Build_244_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  case FT_532: 
  //  Build_532_ReceiverList(m_TempReceiverList,position,LocationSide); 
    return m_TempReceiverList;
  default: 
 
    return  m_ReceiverList[position]; 
  }
}


CUnit *CFormation::GetUnit(int num)
{
  return &m_FormationUnits[num];
}


bool CFormation::IsUnitMember(int unit, Pnum position)
{
  return GetUnit(unit)->IsMember(position);
}


bool CFormation::IsUnitCaptain(int unit, Pnum position)
{
  return GetUnit(unit)->IsCaptain(position);
}

bool CFormation::IsMoreImportantPosition(Pnum pos1, Pnum pos2)
{
//  if ( pos1 == pos2 )
    //my_error("same positions more important???");

  for (int unit=0; unit<m_nNumUnits; unit++){
    if ( IsUnitMember(unit,pos1) && IsUnitMember(unit,pos2) ){
      if ( GetUnit(unit)->ImportantPosition(pos1,pos2) == pos1 )
	return TRUE;
    }
  }
  return FALSE;
}


Pnum CFormation::LocationToPosition(C2DVector p)
{
  /* Finds the position whose home range (x,y) falls.  If more than one,
     takes the one closest to the home position.  If none, takes the one
     whose home range rectangle is closest */

  double tiebreakDist = -1000;  /* > 2 * (pitch_length + pitch_width) */
  Pnum position;
  RRectangle *HomeRange;

  int FIXME_formacie_boli_vyripovane;

// FIXME:
//  for (int i=1; i<=GetSize(); i++){
//  HomeRange = GetPosition(i)->GetHomeRange();
//  if ( HomeRange->IsWithin(p) ){  /* Check if (x,y) is within the home range */
//    double HomeDist = DistanceToPositionHome(i,p);
//    if ( tiebreakDist < 0 || HomeDist < tiebreakDist ){
//      position = i;
//      tiebreakDist = HomeDist;
//    }
//  }    /* We're not in the range of this position. Now check if closest so far */
//  else if ( tiebreakDist < 0 && HomeRange->DistanceToEdge(p) > tiebreakDist ){
//    position = i;
//    tiebreakDist = HomeRange->DistanceToEdge(p);    /* a negative number     */
//  }
//}

//return position;
  return 0;
}


double CFormation::DistanceToPositionHome(Pnum position, C2DVector p)
{
  C2DVector Home = GetPosition(position)->GetHome();

  int FIXME_formacie_boli_vyripovane;

// FIXME:
//  return Home.dist(p);
  return 0;
}


Pnum CFormation::ClosestPosition(C2DVector p)
{
  double dist, minDist = HUGE;
  Pnum closestPosition;
  for (int position=1; position<=m_size; position++){
    dist = DistanceToPositionHome(position,p);
    if (dist < minDist){
      minDist = dist;
      closestPosition = position;
    }
  }
  return closestPosition;
}


int CFormation::Build_433_ReceiverList(Pnum *PreferenceList, Pnum position, Fside LocationSide){

  int NumOptions=0;

  switch ( position ){
  case f433_CENTER_FORWARD: 
    NumOptions = 2;
    PreferenceList[0] = (LocationSide == FS_Right) ? f433_RIGHT_WING : f433_LEFT_WING;
    PreferenceList[1] = (LocationSide == FS_Right) ? f433_LEFT_WING : f433_RIGHT_WING;
  case f433_LEFT_WING:
    NumOptions = 2;
    PreferenceList[0] = f433_CENTER_FORWARD;
    PreferenceList[1] = f433_RIGHT_WING;
  case f433_RIGHT_WING:
    NumOptions = 2;
    PreferenceList[0] = f433_CENTER_FORWARD;
    PreferenceList[1] = f433_LEFT_WING;
    break;
  case f433_CENTER_MIDFIELD:
    NumOptions = 5;
    PreferenceList[0] = (LocationSide == FS_Right) ? f433_RIGHT_WING : f433_LEFT_WING;
    PreferenceList[1] = (LocationSide == FS_Right) ? f433_LEFT_WING : f433_RIGHT_WING;
    PreferenceList[2] = f433_CENTER_FORWARD;
    PreferenceList[3] = (LocationSide == FS_Right) ? f433_RIGHT_MIDFIELD :f433_LEFT_MIDFIELD;
    PreferenceList[4] = (LocationSide == FS_Right) ? f433_LEFT_MIDFIELD :f433_RIGHT_MIDFIELD;
    break;
  case f433_RIGHT_MIDFIELD:
  case f433_LEFT_MIDFIELD:
    NumOptions = 4;
    PreferenceList[0] = f433_CENTER_FORWARD;
    PreferenceList[1] = f433_CENTER_MIDFIELD;
    PreferenceList[2] = (LocationSide == FS_Right) ? f433_RIGHT_WING : f433_LEFT_WING;
    PreferenceList[3] = (LocationSide == FS_Right) ? f433_LEFT_WING : f433_RIGHT_WING;
    break;
  case f433_CENTER_DEFENSE:    
    NumOptions = 3;
    PreferenceList[0] = (LocationSide == FS_Right) ? f433_RIGHT_MIDFIELD : f433_LEFT_MIDFIELD;
    PreferenceList[1] = (LocationSide == FS_Right) ? f433_LEFT_MIDFIELD : f433_RIGHT_MIDFIELD;
    PreferenceList[2] = f433_CENTER_MIDFIELD;
    break;
  case f433_RIGHT_DEFENSE:
  case f433_LEFT_DEFENSE:
    NumOptions = 3;
    PreferenceList[0] = (LocationSide == FS_Right) ? f433_RIGHT_MIDFIELD : f433_LEFT_MIDFIELD;
    PreferenceList[1] = (LocationSide == FS_Right) ? f433_RIGHT_WING : f433_LEFT_WING;
    PreferenceList[2] = f433_CENTER_MIDFIELD;
    break;
  case f433_SWEEPER:
  case f433_GOALIE:
    NumOptions = 7;
    PreferenceList[0] = (LocationSide == FS_Right) ? f433_RIGHT_DEFENSE : f433_LEFT_DEFENSE;
    PreferenceList[1] = (LocationSide == FS_Right) ? f433_LEFT_DEFENSE : f433_RIGHT_DEFENSE;
    PreferenceList[2] = (LocationSide == FS_Right) ? f433_RIGHT_MIDFIELD : f433_LEFT_MIDFIELD;
    PreferenceList[3] = (LocationSide == FS_Right) ? f433_LEFT_MIDFIELD : f433_RIGHT_MIDFIELD;
    PreferenceList[4] = f433_CENTER_MIDFIELD;
    PreferenceList[5] = (LocationSide == FS_Right) ? f433_RIGHT_WING : f433_LEFT_WING;
    PreferenceList[6] = (LocationSide == FS_Right) ? f433_LEFT_WING : f433_RIGHT_WING;
    break;
  }	

  for (int i = NumOptions; i<m_size; i++)
    PreferenceList[i] = Pnum_Unknown;  /* Pad list with own number, meaning no receiver */

  return NumOptions;
}


int CFormation::Build_442_ReceiverList(Pnum *PreferenceList, Pnum position, Fside LocationSide){

  int NumOptions=0;
  
  Pside Side = GetPosition(position)->GetSide();

  switch ( position ){
  case f442_LEFT_WING:
  case f442_RIGHT_WING:
    NumOptions = (rand() ? 1 : 0); /* shoot 1 in 3 times  int_random(3) */
    PreferenceList[0] =  (Side == PS_Right) ? f442_LEFT_WING : f442_RIGHT_WING;
    break;
  case f442_OFFENSIVE_CENTER_MIDFIELD:
    NumOptions = 2;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_WING : f442_LEFT_WING;
    PreferenceList[1] = (LocationSide == FS_Right) ? f442_LEFT_WING : f442_RIGHT_WING;
    break;
  case f442_DEFENSIVE_CENTER_MIDFIELD:
    NumOptions = 5;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_WING : f442_LEFT_WING;
    PreferenceList[1] = (LocationSide == FS_Right) ? f442_LEFT_WING : f442_RIGHT_WING;
    PreferenceList[2] = f442_OFFENSIVE_CENTER_MIDFIELD;
    PreferenceList[3] = (LocationSide == FS_Right) ? f442_RIGHT_MIDFIELD :f442_LEFT_MIDFIELD;
    PreferenceList[4] = (LocationSide == FS_Right) ? f442_LEFT_MIDFIELD :f442_RIGHT_MIDFIELD;
    break;
  case f442_RIGHT_MIDFIELD:
  case f442_LEFT_MIDFIELD:
    NumOptions = 4;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_WING : f442_LEFT_WING;
    PreferenceList[1] = f442_OFFENSIVE_CENTER_MIDFIELD;
    PreferenceList[2] = (LocationSide == FS_Right) ? f442_LEFT_WING : f442_RIGHT_WING;
    PreferenceList[3] = f442_DEFENSIVE_CENTER_MIDFIELD;
    break;
  case f442_CENTER_DEFENSE:    
    NumOptions = 4;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_MIDFIELD : f442_LEFT_MIDFIELD;
    PreferenceList[1] = (LocationSide == FS_Right) ? f442_LEFT_MIDFIELD : f442_RIGHT_MIDFIELD;
    PreferenceList[2] = f442_OFFENSIVE_CENTER_MIDFIELD;
    PreferenceList[3] = f442_DEFENSIVE_CENTER_MIDFIELD;
    break;
  case f442_RIGHT_DEFENSE:
  case f442_LEFT_DEFENSE:
    NumOptions = 4;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_MIDFIELD : f442_LEFT_MIDFIELD;
    PreferenceList[1] = (LocationSide == FS_Right) ? f442_RIGHT_WING : f442_LEFT_WING;
    PreferenceList[2] = f442_OFFENSIVE_CENTER_MIDFIELD;
    PreferenceList[3] = f442_DEFENSIVE_CENTER_MIDFIELD;
    break;
  case f442_SWEEPER:
  case f442_GOALIE:
    NumOptions = 8;
    PreferenceList[0] = (LocationSide == FS_Right) ? f442_RIGHT_DEFENSE : f442_LEFT_DEFENSE;
    PreferenceList[1] = (LocationSide == FS_Right) ? f442_LEFT_DEFENSE : f442_RIGHT_DEFENSE;
    PreferenceList[2] = (LocationSide == FS_Right) ? f442_RIGHT_MIDFIELD : f442_LEFT_MIDFIELD;
    PreferenceList[3] = (LocationSide == FS_Right) ? f442_LEFT_MIDFIELD : f442_RIGHT_MIDFIELD;
    PreferenceList[4] = f442_OFFENSIVE_CENTER_MIDFIELD;
    PreferenceList[5] = f442_DEFENSIVE_CENTER_MIDFIELD;
    PreferenceList[6] = (LocationSide == FS_Right) ? f442_RIGHT_WING : f442_LEFT_WING;
    PreferenceList[7] = (LocationSide == FS_Right) ? f442_LEFT_WING : f442_RIGHT_WING;
    break;
  }	

  for (int i = NumOptions; i<GetSize(); i++)
    PreferenceList[i] = Pnum_Unknown;  /* Pad list with own number, meaning no receiver */

  return NumOptions;
}

